/*
	This file is distributed under the terms of the GNU Public License, version 2.
	
	Copyright (c) Stijn Buys, 2009
*/

#include <windows.h>
#include <wingdi.h>

#include "GL/gl.h"

#include <cmath>

#include "clockflat.h"

/// MSVC does not define M_PI
#ifndef M_PI
#define M_PI           3.14159265358979323846  /* pi */
#endif

namespace glock {

inline const float deg_to_rad(const float angle)
{
	return (angle * (float) M_PI / 180.0f);
}

ClockFlat::ClockFlat(const Context* context) : Clock(context)
{

	for (size_t i = 0; i < 60; i ++) {
		float rad = deg_to_rad((float) i * 6.0f);
		sin_table[i] = ::sinf(rad);
		cos_table[i] = ::cosf(rad);
	}
	
	//glEnable(GL_LINE_SMOOTH);
	//glEnable(GL_POLYGON_SMOOTH);
	
	//glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	//glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
	
	//glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
	//glHint(GL_POLYGON_SMOOTH_HINT, GL_DONT_CARE);

}

ClockFlat::~ClockFlat()
{
	glDisable(GL_LINE_SMOOTH);
}

void ClockFlat::draw_tick(float angle, float width, float from_radius, float to_radius)
{
	glPushMatrix();
	glRotatef(angle, 0.0f, 0.0f, 1.0f);
	
	glBegin(GL_QUADS);
	glVertex2f(from_radius, width * 0.5f);
	glVertex2f(from_radius, -width * 0.5f);
	glVertex2f(to_radius, -width * 0.5f);
	glVertex2f(to_radius, width * 0.5f);
	glEnd();
	
	glPopMatrix();
}

void ClockFlat::draw_disc(float width)
{
	glBegin(GL_POLYGON);
	for (size_t i = 0; i < 60; i++) {
		glVertex2f(cos_table[i] * width, sin_table[i] * width);
	}
	glEnd();
}

void ClockFlat::draw_hand(float angle, float width, float length, bool disc)
{
	if (disc)
		draw_disc(width * 1.3f);
	draw_tick(angle, width, 0, length);
}

void ClockFlat::draw()
{
	// Clear the color and depth buffers.
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	// load identity matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
		
	// move (0,0) to the centre of the viewport
	glTranslatef(context()->width() * 0.5f, context()->height()*0.5f, 0.0f);
	glRotatef(90.0f, 0.0f, 0.0f, -1.0f);
	
	// rescale so we can draw in a 2000x2000 units square
	float min = (float) (context()->width() < context()->height() ? context()->width() : context()->height());
	const float diameter = 0.85f;
	const float s = diameter * min / 2000.0f;
	glScalef(s, s, s);
	
	// draw clock face
	for (size_t i = 0; i < 60; i++) {
		if (i % 5) {
			// minor tick
			glColor3f(0.7f, 0.7f, 0.7f);
			draw_tick(i * 6.0f, 15.0f, 920.0f, 980.0f);
		} else {
			// major tick
			glColor3f(0.85f, 0.85f, 0.85f);
			draw_tick(i * 6.0f, 40.0f, 825.0f, 980.0f);
		}
	}
	
	// draw hours indicator
	glColor3f(1.0f, 1.0f, 1.0f);
	const float deg_hours =  ((float) (hours() % 12) * 360.0f / 12.0f) + ((float)  minutes() * 6.0f / 12.0f);
	draw_hand(deg_hours, 55.0f, 600.0f, false);
	
	// draw minutes indicator
	glColor3f(1.0f, 1.0f, 1.0f);
	const float deg_minutes = ((float) minutes() * 6.0f ) + ((float) seconds() * 6.0f / 60.0f);
	draw_hand(deg_minutes, 40.0f, 900.0f, true);
	
	// draw seconds indicator
	glColor3f(1.0f, 0.0f, 0.0f);
	const float deg_seconds = (float) seconds() * 6.0f;
	draw_hand(deg_seconds, 30.0f, 900.0f, true);	
}

} // namespace glock

